@@ -40,8 +40,8 @@
 
 static gboolean
 affile_open_file(gchar *name, gint flags,
-                 gint uid, gint gid, gint mode,
-                 gint dir_uid, gint dir_gid, gint dir_mode,
+                 uid_t uid, gid_t gid, mode_t mode,
+                 uid_t dir_uid, gid_t dir_gid, mode_t dir_mode,
                  gboolean create_dirs, gboolean privileged, gboolean is_pipe, gint *fd)
 {
   cap_t saved_caps;
@@ -69,17 +69,15 @@ affile_open_file(gchar *name, gint flags,
     {
       if (is_pipe && !S_ISFIFO(st.st_mode))
         {
-          msg_error("Error opening pipe, underlying file is not a FIFO, it should be used by file()",
+          msg_warning("WARNING: you are using the pipe driver, underlying file is not a FIFO, it should be used by file()",
                     evt_tag_str("filename", name),
                     NULL);
-          goto exit;
         }
       else if (!is_pipe && S_ISFIFO(st.st_mode))
         {
-          msg_error("Error opening file, underlying file is a FIFO, it should be used by pipe()",
-                    evt_tag_str("filename", name),
-                    NULL);
-          goto exit;
+          msg_warning("WARNING: you are using the file driver, underlying file is a FIFO, it should be used by pipe()",
+                      evt_tag_str("filename", name),
+                      NULL);
         }
     }
   *fd = open(name, flags, mode);
@@ -95,14 +93,13 @@ affile_open_file(gchar *name, gint flags,
       
       g_process_cap_modify(CAP_CHOWN, TRUE);
       g_process_cap_modify(CAP_FOWNER, TRUE);
-      if (uid >= 0)
-        fchown(*fd, (uid_t) uid, -1);
-      if (gid >= 0)
-        fchown(*fd, -1, (gid_t) gid);
-      if (mode >= 0)
-        fchmod(*fd, (mode_t) mode);
+      if (uid != (uid_t) -1)
+        fchown(*fd, uid, -1);
+      if (gid != (gid_t) -1)
+        fchown(*fd, -1, gid);
+      if (mode != (mode_t) -1)
+        fchmod(*fd, mode);
     }
- exit:
   g_process_cap_restore(saved_caps);
   msg_trace("affile_open_file",
             evt_tag_str("path", name),
@@ -260,8 +257,12 @@ static void
 affile_sd_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options)
 {
   AFFileSourceDriver *self = (AFFileSourceDriver *) s;
+  static NVHandle filename_handle = 0;
+
+  if (!filename_handle)
+    filename_handle = log_msg_get_value_handle("FILE_NAME");
   
-  log_msg_add_dyn_value(msg, "FILE_NAME", self->filename->str);
+  log_msg_set_value(msg, filename_handle, self->filename->str, self->filename->len);
 
   log_pipe_forward_msg(s, msg, path_options);
 }
@@ -763,13 +764,13 @@ affile_dd_init(LogPipe *s)
     self->file_uid = cfg->file_uid;
   if (self->file_gid == -1)
     self->file_gid = cfg->file_gid;
-  if (self->file_perm == -1)
+  if (self->file_perm == (mode_t) -1)
     self->file_perm = cfg->file_perm;
   if (self->dir_uid == -1)
     self->dir_uid = cfg->dir_uid;
   if (self->dir_gid == -1)
     self->dir_gid = cfg->dir_gid;
-  if (self->dir_perm == -1)
+  if (self->dir_perm == (mode_t) -1)
     self->dir_perm = cfg->dir_perm;
   if (self->time_reap == -1)
     self->time_reap = cfg->time_reap;
