@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2007 BalaBit IT Ltd, Budapest, Hungary                    
+ * Copyright (c) 2002-2009 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -20,7 +20,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-
+  
 #ifndef SDFILE_H_INCLUDED
 #define SDFILE_H_INCLUDED
 
@@ -33,17 +33,23 @@
 #define AFFILE_TMPL_ESCAPE 0x00000004
 #define AFFILE_CREATE_DIRS 0x00000008
 #define AFFILE_FSYNC       0x00000010
+#define AFFILE_PRIVILEGED  0x00000020
 
 typedef struct _AFFileSourceDriver
 {
   LogDriver super;
   GString *filename;
+  /* FIXME: the code assumes that reader is a LogReader at a lot of places, so this should be changed to LogReader */
   LogPipe *reader;
   LogReaderOptions reader_options;
   guint32 flags;
+  /* state information to follow a set of files using a wildcard expression */
 } AFFileSourceDriver;
 
 LogDriver *affile_sd_new(gchar *filename, guint32 flags);
+void affile_sd_set_recursion(LogDriver *s, const gint recursion);
+void affile_sd_set_pri_level(LogDriver *s, const gint16 severity);
+void affile_sd_set_pri_facility(LogDriver *s, const gint16 facility);
 
 typedef struct _AFFileDestWriter AFFileDestWriter;
 
@@ -59,10 +65,11 @@ typedef struct _AFFileDestDriver
   uid_t dir_uid;
   gid_t dir_gid;
   mode_t dir_perm;
+  gchar *local_time_zone;
+  TimeZoneInfo *local_time_zone_info;
   LogWriterOptions writer_options;
   GHashTable *writer_hash;
-  GlobalConfig *cfg;
-  
+    
   gint overwrite_if_older;
   gboolean use_time_recvd;
   gint time_reap;
@@ -82,5 +89,6 @@ void affile_dd_set_dir_perm(LogDriver *s, mode_t dir_perm);
 void affile_dd_set_create_dirs(LogDriver *s, gboolean create_dirs);
 void affile_dd_set_fsync(LogDriver *s, gboolean enable);
 void affile_dd_set_overwrite_if_older(LogDriver *s, gint overwrite_if_older);
+void affile_dd_set_local_time_zone(LogDriver *s, const gchar *local_time_zone);
 
 #endif
