@@ -47,7 +47,9 @@ log_parser_process(LogParser *self, LogMessage *msg)
 
   if (G_LIKELY(!self->template))
     {
-      success = self->process(self, msg, msg->message);
+      NVTable *payload = nv_table_ref(msg->payload);
+      success = self->process(self, msg, log_msg_get_value(msg, LM_V_MESSAGE, NULL));
+      nv_table_unref(payload);
     }
   else
     {
@@ -243,22 +245,19 @@ log_csv_parser_process(LogParser *s, LogMessage *msg, const gchar *input)
                 len--;
             }
           if (self->null_value && strncmp(src, self->null_value, len) == 0)
-            log_msg_add_dyn_value(msg, (gchar *) cur_column->data, "");
+            log_msg_set_value(msg, log_msg_get_value_handle((gchar *) cur_column->data), "", 0);
           else
-            log_msg_add_sized_dyn_value(msg, (gchar *) cur_column->data, src, len);
+            log_msg_set_value(msg, log_msg_get_value_handle((gchar *) cur_column->data), src, len);
             
-          src = (gchar *) delim;
-          if (*src)
-            src++;
+          if (*delim == 0)
+            break;
+          src = (gchar *) delim + 1;
           cur_column = cur_column->next;
 
           if (cur_column && cur_column->next == NULL && self->flags & LOG_CSV_PARSER_GREEDY)
             {
-              /* greedy mode, the last column gets it all, without taking escaping, quotes or anything into account */
-              log_msg_add_dyn_value(msg, (gchar *) cur_column->data, src);
+              log_msg_set_value(msg, log_msg_get_value_handle((gchar *) cur_column->data), src, -1);
               cur_column = NULL;
-              src = NULL;
-              break;
             }
         }
     }
@@ -360,9 +359,9 @@ log_csv_parser_process(LogParser *s, LogMessage *msg, const gchar *input)
                     len--;
                 }
               if (self->null_value && strcmp(current_value->str, self->null_value) == 0)
-                log_msg_add_dyn_value(msg, (gchar *) cur_column->data, "");
+                log_msg_set_value(msg, log_msg_get_value_handle((gchar *) cur_column->data), "", 0);
               else
-                log_msg_add_sized_dyn_value(msg, (gchar *) cur_column->data, current_value->str, len);
+                log_msg_set_value(msg, log_msg_get_value_handle((gchar *) cur_column->data), current_value->str, len);
               g_string_truncate(current_value, 0);
               cur_column = cur_column->next;
               state = PS_COLUMN_START;
@@ -370,22 +369,15 @@ log_csv_parser_process(LogParser *s, LogMessage *msg, const gchar *input)
 
               if (cur_column && cur_column->next == NULL && self->flags & LOG_CSV_PARSER_GREEDY)
                 {
-                  /* greedy mode, the last column gets it all, without taking escaping, quotes or anything into account */
-                  log_msg_add_dyn_value(msg, (gchar *) cur_column->data, src);
+                  log_msg_set_value(msg, log_msg_get_value_handle((gchar *) cur_column->data), src, -1);
                   cur_column = NULL;
-                  src = NULL;
-                  break;
                 }
             }
         }
       g_string_free(current_value, TRUE);
     }
-  if ((cur_column || (src && *src)) && (self->flags & LOG_CSV_PARSER_DROP_INVALID))
-    {
-      /* there are unfilled variables, OR not all of the input was processed
-       * and "drop-invalid" flag is specified */
-      return FALSE;
-    }
+  if (!cur_column && (self->flags & LOG_CSV_PARSER_DROP_INVALID))
+    return FALSE;
   return TRUE;
 }
 
@@ -485,11 +477,49 @@ log_db_parser_reload_database(LogDBParser *self)
 
 }
 
+NVHandle class_handle = 0;
+NVHandle rule_id_handle = 0;
+
+static inline void
+log_db_parser_process_real(LogPatternDatabase *db, LogMessage *msg, GSList **dbg_list)
+{
+  LogDBResult *verdict;
+  gint i;
+
+  verdict = log_pattern_database_lookup(db, msg, dbg_list);
+  if (verdict)
+    {
+      log_msg_set_value(msg, class_handle, verdict->class, -1);
+      log_msg_set_value(msg, rule_id_handle, verdict->rule_id, -1);
+
+      if (verdict->tags)
+        {
+          for (i = 0; i < verdict->tags->len; i++)
+            log_msg_set_tag_by_id(msg, g_array_index(verdict->tags, guint, i));
+        }
+
+      if (verdict->values)
+        {
+          GString *result = g_string_sized_new(32);
+
+          for (i = 0; i < verdict->values->len; i++)
+            {
+              log_template_format(g_ptr_array_index(verdict->values, i), msg, 0, TS_FMT_ISO, NULL, 0, 0, result);
+              log_msg_set_value(msg, log_msg_get_value_handle(((LogTemplate *)g_ptr_array_index(verdict->values, i))->name), result->str, result->len);
+            }
+          g_string_free(result, TRUE);
+        }
+    }
+  else
+    {
+      log_msg_set_value(msg, class_handle, "unknown", 7);
+    }
+}
+
 static gboolean
 log_db_parser_process(LogParser *s, LogMessage *msg, const char *input)
 {
   LogDBParser *self = (LogDBParser *) s;
-  LogDBResult *verdict;
 
   if (G_UNLIKELY(self->db_file_last_check == 0 || self->db_file_last_check < msg->timestamps[LM_TS_RECVD].time.tv_sec - 5))
     {
@@ -497,19 +527,16 @@ log_db_parser_process(LogParser *s, LogMessage *msg, const char *input)
       log_db_parser_reload_database(self);
     }
 
-  verdict = log_pattern_database_lookup(&self->db, msg);
-  if (verdict)
-    {
-      log_msg_add_dyn_value(msg, ".classifier.class", verdict->class);
-      log_msg_add_dyn_value(msg, ".classifier.rule_id", verdict->rule_id);
-    }
-  else
-    {
-      log_msg_add_dyn_value(msg, ".classifier.class", "unknown");
-    }
+  log_db_parser_process_real(&self->db, msg, NULL);
   return TRUE;
 }
 
+void
+log_db_parser_process_lookup(LogPatternDatabase *db, LogMessage *msg, GSList **dbg_list)
+{
+  log_db_parser_process_real(db, msg, dbg_list);
+}
+
 void
 log_db_parser_set_db_file(LogDBParser *self, const gchar *db_file)
 {
@@ -588,3 +615,9 @@ log_parser_rule_new(const gchar *name, LogParser *parser)
   return &self->super;
 }
 
+void
+log_db_parser_global_init(void)
+{
+  class_handle = log_msg_get_value_handle(".classifier.class");
+  rule_id_handle = log_msg_get_value_handle(".classifier.rule_id");
+}
