@@ -65,9 +65,9 @@ static gboolean syntax_only = FALSE;
 static gboolean seed_rng = FALSE;
 static gboolean display_version = FALSE;
 
-static volatile sig_atomic_t sig_hup_received = FALSE;
-static volatile sig_atomic_t sig_term_received = FALSE;
-static volatile sig_atomic_t sig_child_received = FALSE;
+static gboolean sig_hup_received = FALSE;
+static gboolean sig_term_received = FALSE;
+static gboolean sig_child_received = FALSE;
 
 static void 
 sig_hup_handler(int signo)
@@ -138,8 +138,6 @@ main_loop_run(GlobalConfig **cfg)
 {
   gint iters;
   guint stats_timer_id = 0;
-  sigset_t ss;
-
   msg_notice("syslog-ng starting up", 
              evt_tag_str("version", VERSION),
              NULL);
@@ -165,25 +163,12 @@ main_loop_run(GlobalConfig **cfg)
       g_main_context_iteration(g_main_loop_get_context(main_loop), TRUE);
       if (sig_hup_received)
         {
-          sigemptyset(&ss);
-          sigaddset(&ss, SIGHUP);
-          sigprocmask(SIG_BLOCK, &ss, NULL);
-          sig_hup_received = FALSE;
-
-          /* this may handle multiple SIGHUP signals, however it doesn't
-           * really matter if we received only a single or multiple SIGHUPs
-           * until we make sure that we handle the last one.  Since we
-           * blocked the SIGHUP signal and reset sig_hup_received to FALSE,
-           * we can be sure that if we receive an additional SIGHUP during
-           * signal processing we get the new one when we finished this, and
-           * handle that one as well. */
-
 	  app_pre_config_loaded();
           (*cfg) = cfg_reload_config(cfgfilename, (*cfg));
 	  app_post_config_loaded();
           msg_notice("Configuration reload request received, reloading configuration", 
                        NULL);
-          reset_cached_hostname();
+          sig_hup_received = FALSE;
           if ((*cfg)->stats_freq > 0)
             {
               if (stats_timer_id != 0)
@@ -191,7 +176,6 @@ main_loop_run(GlobalConfig **cfg)
               stats_timer_id = g_timeout_add((*cfg)->stats_freq * 1000, stats_timer, NULL);
             }
           stats_cleanup_orphans();
-          sigprocmask(SIG_UNBLOCK, &ss, NULL);
         }
       if (sig_term_received)
         {
@@ -204,22 +188,13 @@ main_loop_run(GlobalConfig **cfg)
 	  pid_t pid;
 	  int status;
 
-          sigemptyset(&ss);
-          sigaddset(&ss, SIGCHLD);
-          sigprocmask(SIG_BLOCK, &ss, NULL);
-	  sig_child_received = FALSE;
-
-	  /* this may handle multiple SIGCHLD signals, however it doesn't
-	   * matter if one or multiple SIGCHLD was received assuming that
-	   * all exited child process are waited for */
-
           do
 	    {
 	      pid = waitpid(-1, &status, WNOHANG);
 	      child_manager_sigchild(pid, status);
 	    }
           while (pid > 0);
-          sigprocmask(SIG_UNBLOCK, &ss, NULL);
+	  sig_child_received = FALSE;
 	}
     }
   control_destroy();
@@ -437,16 +412,6 @@ main(int argc, char *argv[])
       return 0;
     }
 
-  if (debug_flag)
-    {
-      log_stderr = TRUE;
-    }
-
-  if (syntax_only || debug_flag)
-    {
-      g_process_set_mode(G_PM_FOREGROUND);
-    }
-
   g_process_set_name("syslog-ng");
   
   /* in this case we switch users early while retaining a limited set of
