@@ -41,9 +41,6 @@
 #include <stdio.h>
 #include <signal.h>
 
-static gchar local_hostname_fqdn[256];
-static gchar local_hostname_short[256];
-
 GString *
 g_string_assign_len(GString *s, const gchar *val, gint len)
 {
@@ -53,35 +50,34 @@ g_string_assign_len(GString *s, const gchar *val, gint len)
   return s;
 }
 
-void
-reset_cached_hostname(void)
+char *
+getshorthostname(char *buf, size_t bufsize)
 {
-  gchar *s;
+  char *s;
   
-  gethostname(local_hostname_fqdn, sizeof(local_hostname_fqdn) - 1);
-  local_hostname_fqdn[sizeof(local_hostname_fqdn) - 1] = '\0';
-  if (strchr(local_hostname_fqdn, '.') == NULL)
-    {
-      /* not fully qualified, resolve it using DNS or /etc/hosts */
-      struct hostent *result = gethostbyname(local_hostname_fqdn);
-      if (result)
-        {
-          strncpy(local_hostname_fqdn, result->h_name, sizeof(local_hostname_fqdn) - 1);
-          local_hostname_fqdn[sizeof(local_hostname_fqdn) - 1] = '\0';
-        }
-    }
-  /* NOTE: they are the same size, they'll fit */
-  strcpy(local_hostname_short, local_hostname_fqdn);
-  s = strchr(local_hostname_short, '.');
+  gethostname(buf, bufsize - 1);
+  buf[bufsize - 1] = '\0';
+  s = strchr(buf, '.');
   if (s != NULL)
     *s = '\0';
+  return buf;
 }
 
-void
-getlonghostname(gchar *buf, gsize buflen)
+char *
+getlonghostname(char *buf, size_t bufsize)
 {
-  strncpy(buf, local_hostname_fqdn, buflen);
-  buf[buflen - 1] = 0;
+  gethostname(buf, bufsize - 1);
+  buf[bufsize - 1] = '\0';
+  if (strchr(buf, '.') == NULL)
+    {
+      struct hostent *result = gethostbyname(buf);
+      if (result)
+        {
+          strncpy(buf, result->h_name, bufsize - 1);
+          buf[bufsize - 1] = 0;
+        }
+    }
+  return buf;
 }
 
 gboolean
@@ -157,9 +153,10 @@ resolve_hostname(GSockAddr **addr, gchar *name)
   return TRUE;
 }
 
-gboolean
-resolve_sockaddr(gchar **result, GSockAddr *saddr, gboolean usedns, gboolean usefqdn, gboolean use_dns_cache, gboolean normalize_hostnames)
+void
+resolve_sockaddr(gchar *result, gsize *result_len, GSockAddr *saddr, gboolean usedns, gboolean usefqdn, gboolean use_dns_cache, gboolean normalize_hostnames)
 {
+  static gchar local_hostname[256] = "";
   gchar *hname;
   gchar *p, buf[256];
  
@@ -225,20 +222,35 @@ resolve_sockaddr(gchar **result, GSockAddr *saddr, gboolean usedns, gboolean use
     }
   else 
     {
-      if (!local_hostname_fqdn[0])
-        reset_cached_hostname();
-      if (usefqdn)
+      hname = local_hostname;
+      
+      if (!local_hostname[0]) 
+	{
+          if (usefqdn)
+            getlonghostname(local_hostname, sizeof(local_hostname));
+          else
+	    getshorthostname(local_hostname, sizeof(local_hostname));
+
+	}
+
+    }
+  if (normalize_hostnames)
+    {
+      gint i;
+
+      for (i = 0; hname[i] && i < (*result_len); i++)
         {
-          /* avoid copy */
-          hname = local_hostname_fqdn;
+          result[i] = g_ascii_tolower(hname[i]);
         }
-      else
-        {
-          hname = local_hostname_short;
-        }
+      *result_len = i;
+    }
+  else
+    {
+      gsize len = g_strlcpy(result, hname, *result_len);
+
+      if (len <= *result_len)
+        *result_len = len;
     }
-  *result = normalize_hostnames ? g_ascii_strdown(hname, -1) : g_strdup(hname);
-  return TRUE;
 }
 
 gboolean
@@ -280,51 +292,57 @@ g_fd_set_cloexec(int fd, gboolean enable)
 }
 
 gboolean 
-resolve_user(const char *user, gint *uid)
+resolve_user(const char *user, uid_t *uid)
 {
   struct passwd *pw;
-  gchar *endptr;
 
   *uid = 0;
   if (!(*user))
     return FALSE;
-
-  *uid = strtol(user, &endptr, 0);
-  if (*endptr)
+    
+  pw = getpwnam(user);
+  if (pw) 
     {
-      pw = getpwnam(user);
-      if (!pw)
-        return FALSE;
-
       *uid = pw->pw_uid;
     }
+  else 
+    {
+      gchar *endptr;
+      
+      *uid = strtol(user, &endptr, 0);
+      if (*endptr)
+        return FALSE;
+    }
   return TRUE;
 }
 
 gboolean 
-resolve_group(const char *group, gint *gid)
+resolve_group(const char *group, gid_t *gid)
 {
   struct group *gr;
-  gchar *endptr;
 
   *gid = 0;
   if (!(*group))
     return FALSE;
     
-  *gid = strtol(group, &endptr, 0);
-  if (*endptr)
+  gr = getgrnam(group);
+  if (gr) 
     {
-      gr = getgrnam(group);
-      if (!gr)
-        return FALSE;
-
       *gid = gr->gr_gid;
     }
+  else 
+    {
+      gchar *endptr;
+      
+      *gid = strtol(group, &endptr, 0);
+      if (*endptr)
+        return FALSE;
+    }
   return TRUE;
 }
 
 gboolean 
-resolve_user_group(char *arg, gint *uid, gint *gid)
+resolve_user_group(char *arg, uid_t *uid, gid_t *gid)
 {
   char *user, *group;
 
@@ -347,11 +365,12 @@ resolve_user_group(char *arg, gint *uid, gint *gid)
  * returns. (at least it won't fail because of missing directories).
  **/
 gboolean
-create_containing_directory(gchar *name, gint dir_uid, gint dir_gid, gint dir_mode)
+create_containing_directory(gchar *name, uid_t dir_uid, gid_t dir_gid, mode_t dir_mode)
 {
   gchar *dirname;
   struct stat st;
   gint rc;
+  gchar *p;
   
   /* check that the directory exists */
   dirname = g_path_get_dirname(name);
@@ -370,7 +389,7 @@ create_containing_directory(gchar *name, gint dir_uid, gint dir_gid, gint dir_mo
     }
     
   /* directory does not exist */
-  char *p = name + 1;
+  p = name + 1;
   
   p = strchr(p, '/');
   while (p) 
@@ -383,14 +402,12 @@ create_containing_directory(gchar *name, gint dir_uid, gint dir_gid, gint dir_mo
         }
       else if (errno == ENOENT) 
         {
-          if (mkdir(name, (mode_t) dir_mode) == -1)
+          if (mkdir(name, dir_mode) == -1)
             return FALSE;
-          if (dir_uid >= 0)
-            chown(name, (uid_t) dir_uid, -1);
-          if (dir_gid >= 0)
-            chown(name, -1, (gid_t) dir_gid);
-          if (dir_mode >= 0)
-            chmod(name, (mode_t) dir_mode);
+          if (dir_uid != -1 || dir_gid != -1)
+            chown(name, dir_uid, dir_gid);
+          if (dir_mode != -1)
+            chmod(name, dir_mode);
         }
       *p = '/';
       p = strchr(p + 1, '/');
@@ -455,8 +472,9 @@ find_cr_or_lf(gchar *s, gsize n)
           ((((longword ^ cr_charmask) + magic_bits) ^ ~(longword ^ cr_charmask)) & ~magic_bits) != 0 || 
           ((((longword ^ lf_charmask) + magic_bits) ^ ~(longword ^ lf_charmask)) & ~magic_bits) != 0)
         {
-          char_ptr = (gchar *) (longword_ptr - 1);
           gint i;
+
+          char_ptr = (gchar *) (longword_ptr - 1);
           
           for (i = 0; i < sizeof(longword); i++)
             {
