@@ -34,7 +34,7 @@
 /* FIXME: maybe we should return gchar with the result */
 
 gboolean
-r_parser_string(gchar *str, gint *len, const gchar *param, gpointer state, LogMessageMatch *match)
+r_parser_string(gchar *str, gint *len, const gchar *param, gpointer state, RParserMatch *match)
 {
   *len = 0;
 
@@ -49,7 +49,7 @@ r_parser_string(gchar *str, gint *len, const gchar *param, gpointer state, LogMe
 }
 
 gboolean
-r_parser_qstring(gchar *str, gint *len, const gchar *param, gpointer state, LogMessageMatch *match)
+r_parser_qstring(gchar *str, gint *len, const gchar *param, gpointer state, RParserMatch *match)
 {
   gchar *end;
 
@@ -71,7 +71,7 @@ r_parser_qstring(gchar *str, gint *len, const gchar *param, gpointer state, LogM
 }
 
 gboolean
-r_parser_estring(gchar *str, gint *len, const gchar *param, gpointer state, LogMessageMatch *match)
+r_parser_estring_c(gchar *str, gint *len, const gchar *param, gpointer state, RParserMatch *match)
 {
   gchar *end;
   
@@ -90,7 +90,33 @@ r_parser_estring(gchar *str, gint *len, const gchar *param, gpointer state, LogM
 }
 
 gboolean
-r_parser_ipv4(gchar *str, gint *len, const gchar *param, gpointer state, LogMessageMatch *match)
+r_parser_estring(gchar *str, gint *len, const gchar *param, gpointer state, RParserMatch *match)
+{
+  gchar *end;
+  
+  if (!param)
+    return FALSE;
+
+  if ((end = strstr(str, param)) != NULL)
+    {
+      *len = (end - str) + GPOINTER_TO_INT(state);
+      if (match)
+        match->len = -GPOINTER_TO_INT(state);
+      return TRUE;
+    }
+  else
+    return FALSE;
+}
+
+gboolean
+r_parser_anystring(gchar *str, gint *len, const gchar *param, gpointer state, RParserMatch *match)
+{
+  *len = strlen(str);
+  return TRUE;
+}
+
+gboolean
+r_parser_ipv4(gchar *str, gint *len, const gchar *param, gpointer state, RParserMatch *match)
 {
   gint dots = 0;
   gint octet = -1;
@@ -104,6 +130,9 @@ r_parser_ipv4(gchar *str, gint *len, const gchar *param, gpointer state, LogMess
           if (octet > 255 || octet == -1)
             return FALSE;
 
+          if (G_UNLIKELY(dots == 3))
+            break;
+
           dots++;
           octet = -1;
         }
@@ -129,14 +158,119 @@ r_parser_ipv4(gchar *str, gint *len, const gchar *param, gpointer state, LogMess
 }
 
 gboolean
-r_parser_number(gchar *str, gint *len, const gchar *param, gpointer state, LogMessageMatch *match)
+r_parser_ipv6(gchar *str, gint *len, const gchar *param, gpointer state, RParserMatch *match)
 {
+  gint colons = 0;
+  gint dots = 0;
+  gint octet = 0;
+  gint digit = 16;
+  gboolean shortened = FALSE;
+
   *len = 0;
 
-  while (g_ascii_isdigit(str[*len]))
+  while (1)
+    {
+      if (str[*len] == ':')
+        {
+          if (G_UNLIKELY(octet > 0xffff || (octet == -1 && shortened) || digit == 10))
+            return FALSE;
+
+          if (octet == -1)
+            shortened = TRUE;
+
+          if (G_UNLIKELY(colons == 7))
+            break;
+
+          colons++;
+          octet = -1;
+        }
+      else if (g_ascii_isxdigit(str[*len]))
+        {
+          if (octet == -1)
+            octet = 0;
+          else
+            octet *= digit;
+
+          octet += g_ascii_xdigit_value(str[*len]);
+        }
+      else if (str[*len] == '.')
+        {
+          if (G_UNLIKELY((digit == 10 && octet > 255)))
+              return FALSE;
+
+          if (G_UNLIKELY((digit == 16 && octet > 597) || octet == -1 || colons == 7 || dots == 3))
+            break;
+
+          dots++;
+          octet = -1;
+          digit = 10;
+        }
+      else
+        break;
+
+      (*len)++;
+    }
+
+  if (G_UNLIKELY(str[*len - 1] == '.'))
+    {
+      (*len)--;
+      dots--;
+    }
+  else if (G_UNLIKELY(str[*len - 1] == ':' && str[*len - 2] != ':'))
+    {
+      (*len)--;
+      colons--;
+    }
+
+  if (colons < 2 || colons > 7 || (digit == 10 && octet > 255) || (digit == 16 && octet > 0xffff) ||
+      !(dots == 0 || dots == 3) || (!shortened && colons < 7 && dots == 0))
+    return FALSE;
+
+  return TRUE;
+}
+
+gboolean
+r_parser_ip(gchar *str, gint *len, const gchar *param, gpointer state, RParserMatch *match)
+{
+  return r_parser_ipv4(str, len, param, state, match) || r_parser_ipv6(str, len, param, state, match);
+}
+
+gboolean
+r_parser_float(gchar *str, gint *len, const gchar *param, gpointer state, RParserMatch *match)
+{
+  gboolean dot = FALSE;
+
+  *len = 1;
+
+  while (g_ascii_isdigit(str[*len]) || (!dot && str[*len] == '.' && (dot = TRUE)))
     (*len)++;
 
-  if (*len > 0)
+  return TRUE;
+}
+
+gboolean
+r_parser_number(gchar *str, gint *len, const gchar *param, gpointer state, RParserMatch *match)
+{
+  gboolean hex = FALSE;
+
+  if (g_str_has_prefix(str, "0x") || g_str_has_prefix(str, "0X"))
+    {
+      *len = 2;
+      hex = TRUE;
+
+      while (g_ascii_isxdigit(str[*len]))
+        (*len)++;
+
+    }
+  else
+    {
+      *len = 0;
+
+      while (g_ascii_isdigit(str[*len]))
+        (*len)++;
+    }
+
+  if ((!hex && *len > 0) || (hex && *len > 2))
     {
       return TRUE;
     }
@@ -155,42 +289,71 @@ r_new_pnode(gchar *key)
   gchar **params = g_strsplit(key, ":", 3);
   guint params_len = g_strv_length(params);
 
-  if (g_str_has_prefix(params[0], "IPv4"))
+  parser_node->first = 0;
+  parser_node->last = 255;
+
+  if (strcmp(params[0], "IPv4") == 0)
     {
       parser_node->parse = r_parser_ipv4;
       parser_node->type = RPT_IPV4;
-      parser_node->mask = '0';
-      parser_node->first = '9' & '0';
+      parser_node->first = '0';
+      parser_node->last = '9';
     }
-  else if (g_str_has_prefix(params[0], "NUMBER"))
+  else if (strcmp(params[0], "IPv6") == 0)
+    {
+      parser_node->parse = r_parser_ipv6;
+      parser_node->type = RPT_IPV6;
+    }
+  else if (strcmp(params[0], "IPvANY") == 0)
+    {
+      parser_node->parse = r_parser_ip;
+      parser_node->type = RPT_IP;
+    }
+  else if (strcmp(params[0], "NUMBER") == 0)
     {
       parser_node->parse = r_parser_number;
       parser_node->type = RPT_NUMBER;
-      parser_node->mask = '0';
-      parser_node->first = '9' & '0';
+      parser_node->first = '0';
+      parser_node->last = '9';
     }
-  else if (g_str_has_prefix(params[0], "STRING"))
+  else if (strcmp(params[0], "FLOAT") == 0 || strcmp(params[0], "DOUBLE") == 0)
+    {
+      /* DOUBLE is a deprecated alias for FLOAT */
+      parser_node->parse = r_parser_float;
+      parser_node->type = RPT_FLOAT;
+      parser_node->first = '0';
+      parser_node->last = '9';
+    }
+  else if (strcmp(params[0], "STRING") == 0)
     {
       parser_node->parse = r_parser_string;
       parser_node->type = RPT_STRING;
-      parser_node->mask = 0;
-      parser_node->first = 0;
     }
-  else if (g_str_has_prefix(params[0], "ESTRING"))
+  else if (strcmp(params[0], "ESTRING") == 0)
     {
-      parser_node->parse = r_parser_estring;
+      parser_node->parse = r_parser_estring_c;
       parser_node->type = RPT_ESTRING;
-      parser_node->mask = 0;
-      parser_node->first = 0;
+
+      if (params[2] && (strlen(params[2]) > 1))
+        {
+          gint len = strlen(params[2]); 
+          parser_node->state = GINT_TO_POINTER(len);
+          parser_node->parse = r_parser_estring;
+        }
     }
-  else if (g_str_has_prefix(params[0], "QSTRING") && params_len == 3)
+  else if (strcmp(params[0], "ANYSTRING") == 0)
+    {
+      parser_node->parse = r_parser_anystring;
+      parser_node->type = RPT_ANYSTRING;
+    }
+  else if (strcmp(params[0], "QSTRING") == 0 && params_len == 3)
     {
       gchar *state = (gchar *) &(parser_node->state);
 
       parser_node->parse = r_parser_qstring;
       parser_node->type = RPT_QSTRING;
-      parser_node->mask = 0xff;
       parser_node->first = params[2][0];
+      parser_node->last = params[2][0];
       
       if (strlen(params[2]) == 2)
         state[0] = params[2][1];
@@ -207,8 +370,8 @@ r_new_pnode(gchar *key)
 
   if (parser_node && params[1])
     {
-      parser_node->name = g_strdup(params[1]);
-      parser_node->name_len = strlen(parser_node->name);
+      if (params[1][0])
+        parser_node->handle = log_msg_get_value_handle(params[1]);
 
       if (params[2])
         parser_node->param = g_strdup(params[2]);
@@ -224,9 +387,6 @@ r_new_pnode(gchar *key)
 void
 r_free_pnode_only(RParserNode *parser)
 {
-  if (parser->name)
-    g_free(parser->name);
-
   if (parser->param)
     g_free(parser->param);
 
@@ -276,7 +436,7 @@ r_add_child(RNode *parent, RNode *child)
 }
 
 static inline void
-r_add_child_check(RNode *root, gchar *key, gpointer value, gboolean parser)
+r_add_child_check(RNode *root, gchar *key, gpointer value, gboolean parser, RNodeGetValueFunc value_func)
 {
   gchar *at;
 
@@ -293,12 +453,12 @@ r_add_child_check(RNode *root, gchar *key, gpointer value, gboolean parser)
 
           /* and insert the rest begining from @ under the newly created literal node */
           *at = '@';
-          r_insert_node(child, at, value, parser);
+          r_insert_node(child, at, value, parser, value_func);
         }
       else
         {
           /* @ is the first so let's insert it simply and let insert_node handle @ */
-          r_insert_node(root, key, value, parser);
+          r_insert_node(root, key, value, parser, value_func);
         }
     }
   else
@@ -323,8 +483,10 @@ gboolean
 r_equal_pnode(RParserNode *a, RParserNode *b)
 {
   return ((a->parse == b->parse) &&
-      ((a->name == NULL && b->name == NULL) ||
-       (a->name != NULL && b->name != NULL && g_str_equal(a->name, b->name))));
+          (a->handle == b->handle) &&
+          ((a->param == NULL && b->param == NULL) ||
+           (a->param != NULL && b->param != NULL && g_str_equal(a->param, b->param)))
+      );
 }
 
 RNode *
@@ -370,7 +532,7 @@ r_find_child(RNode *root, char key)
 }
 
 void
-r_insert_node(RNode *root, gchar *key, gpointer value, gboolean parser)
+r_insert_node(RNode *root, gchar *key, gpointer value, gboolean parser, RNodeGetValueFunc value_func)
 {
   RNode *node;
   gint keylen = strlen(key);
@@ -402,12 +564,15 @@ r_insert_node(RNode *root, gchar *key, gpointer value, gboolean parser)
               if (!node->value)
                 node->value = value;
               else
-                msg_error("Duplicate key in parser radix tree", evt_tag_str("key", "@"), NULL);
+                msg_error("Duplicate key in parser radix tree",
+                    evt_tag_str("key", "@"),
+                    evt_tag_str("value", value_func ? value_func(value) : "unknown"),
+                    NULL);
             }
 
           /* go down building the tree if there is key left */
           if (keylen > 2)
-            r_insert_node(node, key + 2, value, parser);
+            r_insert_node(node, key + 2, value, parser, value_func);
 
         }
       else if ((keylen >= 2) && (end = strchr((const gchar *)key + 1, '@')) != NULL)
@@ -437,7 +602,7 @@ r_insert_node(RNode *root, gchar *key, gpointer value, gboolean parser)
               if ((end - key) < (keylen - 1))
                 {
                   /* the key is not over so go on building the tree */
-                  r_insert_node(node, end + 1, value, parser);
+                  r_insert_node(node, end + 1, value, parser, value_func);
                 }
               else
                 {
@@ -452,7 +617,8 @@ r_insert_node(RNode *root, gchar *key, gpointer value, gboolean parser)
                       /* FIXME: print parser type in string format */
                       msg_error("Duplicate parser node in radix tree",
                                 evt_tag_int("type", node->parser->type),
-                                evt_tag_str("name", (node->parser->name ? node->parser->name : "")),
+                                evt_tag_str("name", log_msg_get_value_name(node->parser->handle, NULL)),
+                                evt_tag_str("value", value_func ? value_func(value) : "unknown"),
                                 NULL);
                     }
                 }
@@ -460,7 +626,10 @@ r_insert_node(RNode *root, gchar *key, gpointer value, gboolean parser)
 
         }
       else
-        msg_error("Key contains '@' without escaping", evt_tag_str("key", key), NULL);
+        msg_error("Key contains '@' without escaping",
+                    evt_tag_str("key", key),
+                    evt_tag_str("value", value_func ? value_func(value) : "unknown"),
+                    NULL);
     }
   else
     {
@@ -486,13 +655,13 @@ r_insert_node(RNode *root, gchar *key, gpointer value, gboolean parser)
             {
               /* @ is always a singel node, and we also have an @ so insert us under root */
               if (parser && key[i] == '@')
-                r_insert_node(root, key + i, value, parser);
+                r_insert_node(root, key + i, value, parser, value_func);
               else
-                r_insert_node(node, key + i, value, parser);
+                r_insert_node(node, key + i, value, parser, value_func);
             }
           else
             {
-              r_add_child_check(root, key + i, value, parser);
+              r_add_child_check(root, key + i, value, parser, value_func);
             }
         }
       else if (i == keylen && i == nodelen)
@@ -502,7 +671,10 @@ r_insert_node(RNode *root, gchar *key, gpointer value, gboolean parser)
           if (!root->value)
             root->value = value;
           else
-            msg_error("Duplicate key in radix tree", evt_tag_str("key", key), NULL);
+            msg_error("Duplicate key in radix tree",
+                        evt_tag_str("key", key),
+                        evt_tag_str("value", value_func ? value_func(value) : "unknown"),
+                        NULL);
         }
       else if (i > 0 && i < nodelen)
         {
@@ -538,7 +710,7 @@ r_insert_node(RNode *root, gchar *key, gpointer value, gboolean parser)
           if (i < keylen)
             {
               /* we add a new sub tree */
-              r_add_child_check(root, key + i, value, parser); 
+              r_add_child_check(root, key + i, value, parser, value_func);
             }
           else
             {
@@ -549,182 +721,15 @@ r_insert_node(RNode *root, gchar *key, gpointer value, gboolean parser)
       else
         {
           /* simply a new children */
-          r_add_child_check(root, key + i, value, parser);
+          r_add_child_check(root, key + i, value, parser, value_func);
         }
     }
 }
 
-
-/* FIXME: a non-recursive algorithm might be faster */
-RNode *
-r_find_node(RNode *root, gchar *whole_key, gchar *key, gint keylen, GArray *matches, GPtrArray *match_names)
-{
-  RNode *node, *ret;
-  gint nodelen = root->keylen;
-  gint j, m;
-  register gint i;
- 
-  if (nodelen < 1)
-    i = 0;
-  else if (nodelen == 1)
-    i = 1;
-  else
-    {
-      m = MIN(keylen, nodelen);
-
-      /* this is a prefix match algorithm, we are interested how long
-       * the common part between key and root->key is. Currently this
-       * uses a byte-by-byte comparison, using a 64/32/16 bit units
-       * would be better. 
-       * 
-       * The code below to perform aligned comparison is commented out
-       * as it does not seem to matter and it is way more complex than
-       * the simple algorithm below. */
-
-#if 0
-      if (2 <= m && (((unsigned long)key % 2) == 0))
-        {
-          gushort *keylong = key;
-          gushort *rootlong = root->key;
-          i = 0;
-
-          while ((i + 2) <= m)
-            {
-              if (*keylong != *rootlong)
-                break;
-
-              i += 2;
-              keylong = key + i;
-              rootlong = root->key + i;
-            }
-          /*printf("RESULT %d\n", i); */
-        }
-      else
-#endif
-        i = 1;
-
-      while (i < m)
-        {
-          if (key[i] != root->key[i])
-            break;
-
-          i++;
-        }
-    }
-
-  msg_trace("Looking up node in the radix tree",
-            evt_tag_int("i", i),
-            evt_tag_int("nodelen", nodelen),
-            evt_tag_int("keylen", keylen),
-            evt_tag_str("root_key", root->key),
-            evt_tag_str("key", key),
-            NULL);
-
-  if (i == keylen && (i == nodelen || nodelen == -1))
-    {
-      if (root->value)
-        return root;
-    }
-  else if ((nodelen < 1) || (i < keylen && i >= nodelen))
-    {
-      ret = NULL;
-      node = r_find_child(root, key[i]);
-
-      if (node)
-        ret = r_find_node(node, whole_key, key + i, keylen - i, matches, match_names);
-
-      /* we only search if there is no match */
-      if (!ret)
-        {
-          gint len;
-          RParserNode *parser_node;
-          gint match_ofs = 0;
-          LogMessageMatch *match = NULL;
-          gchar **match_name;
-
-          if (matches)
-            {
-              match_ofs = matches->len;
-              
-              g_array_set_size(matches, match_ofs + 1);
-              g_ptr_array_set_size(match_names, match_ofs + 1);
-            }
-          for (j = 0; j < root->num_pchildren; j++)
-            {
-              parser_node = root->pchildren[j]->parser;
-
-              if (matches)
-                {
-                  match = &g_array_index(matches, LogMessageMatch, match_ofs);
-                  memset(match, 0, sizeof(*match));
-                  match->flags = LMM_REF_MATCH;
-                }
-              if (((key[i] & parser_node->mask) == parser_node->first) &&
-                  (parser_node->parse(key + i, &len, parser_node->param, parser_node->state, match)))
-                {
-
-                  /* FIXME: we don't try to find the longest match in case
-                   * the radix tree is split on a parser node. The correct
-                   * approach would be to try all parsers and select the
-                   * best match, however it is quite expensive and difficult
-                   * to implement and we don't really expect this to be a
-                   * realistic case. A log message is printed if such a
-                   * collision occurs, so there's a slight chance we'll
-                   * recognize if this happens in real life. */
-                  
-                  ret = r_find_node(root->pchildren[j], whole_key, key + i + len, keylen - (i + len), matches, match_names);
-                  if (matches)
-                    {
-
-                      match = &g_array_index(matches, LogMessageMatch, match_ofs);
-                      match_name = (gchar **) &g_ptr_array_index(match_names, match_ofs);
-
-                      if (ret)
-                        {
-                          if ((match->flags & LMM_REF_MATCH))
-                            {
-                              /* NOTE: we allow the parser to return relative
-                               * offset & length to the field parsed, this way
-                               * quote characters can still be returned as
-                               * REF_MATCH and we only need to duplicate the
-                               * result if the string is indeed modified
-                               */
-                              match->type = parser_node->type;
-                              match->ofs = match->ofs + (key + i) - whole_key;
-                              match->len = match->len + len;
-                            }
-                          if (parser_node->name)
-                            *match_name = g_strndup(parser_node->name, parser_node->name_len);
-                          else
-                            *match_name = NULL;
-                          break;
-                        }
-                      else
-                        {
-                          if ((match->flags & LMM_REF_MATCH) == 0)
-                            {
-                              /* free the stored match, if this was a dead-end */
-                              g_free(match->match);
-                            }
-                        }
-                    }
-                }
-            }
-          if (!ret && matches)
-            {
-              /* the values in the matches array has already been freed if we come here */
-              g_array_set_size(matches, match_ofs);
-            }
-        }
-
-      if (ret)
-        return ret;
-      else if (root->value)
-        return root;
-    }
-
-  return NULL;
-}
+#define RADIX_DBG 1
+#include "radix-find.c"
+#undef RADIX_DBG
+#include "radix-find.c"
 
 /**
  * r_new_node:
