@@ -18,14 +18,12 @@ gboolean verbose = FALSE;
 LogMessage *init_msg(gchar *msg_string, gboolean msg_flags)
 {
   LogMessage *msg = log_msg_new(msg_string, strlen(msg_string), g_sockaddr_inet_new("10.10.10.10", 1010), msg_flags, NULL, -1, 0xFFFF);
-  log_msg_add_dyn_value(msg, "APP.VALUE", "value");
-  msg->matches = g_new0(LogMessageMatch, 2);
-  msg->matches[0].match = g_strdup("whole-match");
-  msg->matches[1].match = g_strdup("first-match");
-  msg->num_matches = 2;
+  log_msg_set_value(msg, log_msg_get_value_handle("APP.VALUE"), "value", 5);
+  log_msg_set_match(msg, 0, "whole-match", 11);
+  log_msg_set_match(msg, 1, "first-match", 11);
 
   /* fix some externally or automatically defined values */
-  log_msg_set_host_from(msg, g_strdup("kismacska"), -1);
+  log_msg_set_value(msg, LM_V_HOST_FROM, "kismacska", 9);
   msg->timestamps[LM_TS_RECVD].time.tv_sec = 1139684315;
   msg->timestamps[LM_TS_RECVD].time.tv_usec = 639000;
   msg->timestamps[LM_TS_RECVD].zone_offset = get_local_timezone_ofs(1139684315);
@@ -51,40 +49,40 @@ testcase(gchar *msg_string, gchar *template, gboolean use_syslog_protocol,gchar
   opt.options = LWO_NO_MULTI_LINE | LWO_NO_STATS | LWO_SHARE_STATS;
   opt.send_time_zone_info=tzinfo;
 
-  if(use_syslog_protocol)
-  {
-    opt.options |= LWO_SYSLOG_PROTOCOL;
-    msg_flags =  LP_SYSLOG_PROTOCOL;
-    writer_flags = LW_SYSLOG_PROTOCOL;
-  }
-  if(template)
-  {
-    templ = log_template_new("dummy", template);
-    log_template_compile(templ,&error);
-  }
+  if (use_syslog_protocol)
+    {
+      opt.options |= LWO_SYSLOG_PROTOCOL;
+      msg_flags =  LP_SYSLOG_PROTOCOL;
+      writer_flags = LW_SYSLOG_PROTOCOL;
+    }
+  if (template)
+    {
+      templ = log_template_new("dummy", template);
+      log_template_compile(templ,&error);
+    }
   opt.template = templ;
   msg = init_msg(msg_string,msg_flags);
   writer = (LogWriter*)log_writer_new(writer_flags);
-  if(writer)
-  {
-    writer->options = &opt;
-    log_writer_format_log(writer,msg,res);
-    if(strcmp(res->str,expected_value)!=0)
+  if (writer)
     {
-      fprintf(stderr,"Testcase failed; result: %s, expected: %s\n",res->str,expected_value);
+      writer->options = &opt;
+      log_writer_format_log(writer,msg,res);
+      if(strcmp(res->str,expected_value)!=0)
+        {
+          fprintf(stderr,"Testcase failed; result: %s, expected: %s\n",res->str,expected_value);
+          exit(1);
+        }
+    }
+  else
+    {
+      fprintf(stderr,"Error: Can't create log_writer\n");
       exit(1);
     }
-  }
-  else
-  {
-    fprintf(stderr,"Error: Can't create log_writer\n");
-    exit(1);
-  }
-  if(templ)
+  if (templ)
     log_template_unref(templ);
-  if(writer)
+  if (writer)
     free(writer);
-  if(msg)
+  if (msg)
     free(msg);
   g_string_free(res, TRUE);
 }
