@@ -23,10 +23,10 @@ testcase(LogMessage *msg, gchar *template, gchar *expected)
 
   if (!tzinfo)
     tzinfo = time_zone_info_new(NULL);
-  
+
   templ = log_template_new("dummy", template);
   log_template_format(templ, msg, LT_ESCAPE, TS_FMT_BSD, tzinfo, 3, 0, res);
-  
+
   if (strcmp(res->str, expected) != 0)
     {
       fprintf(stderr, "FAIL: template test failed, template=%s, [%s] <=> [%s]\n", template, res->str, expected);
@@ -40,33 +40,34 @@ testcase(LogMessage *msg, gchar *template, gchar *expected)
   g_string_free(res, TRUE);
 }
 
-int 
+int
 main(int argc G_GNUC_UNUSED, char *argv[] G_GNUC_UNUSED)
 {
   LogMessage *msg;
   char *msg_str = "<155>2006-02-11T10:34:56+01:00 bzorp syslog-ng[23323]:árvíztűrőtükörfúrógép";
   GlobalConfig dummy;
-  
+
   if (argc > 1)
     verbose = TRUE;
 
   configuration = &dummy;
   dummy.version = 0x0201;
-  
+
   app_startup();
-  
+
   putenv("TZ=MET-1METDST");
   tzset();
 
   msg = log_msg_new(msg_str, strlen(msg_str), g_sockaddr_inet_new("10.10.10.10", 1010), 0, NULL, -1, 0xFFFF);
-  log_msg_add_dyn_value(msg, "APP.VALUE", "value");
-  msg->matches = g_new0(LogMessageMatch, 2);
-  msg->matches[0].match = g_strdup("whole-match");
-  msg->matches[1].match = g_strdup("first-match");
-  msg->num_matches = 2;
-  
+  log_msg_set_value(msg, log_msg_get_value_handle("APP.VALUE"), "value", -1);
+  log_msg_set_match(msg, 0, "whole-match", -1);
+  log_msg_set_match(msg, 1, "first-match", -1);
+  log_msg_set_tag_by_name(msg, "alma");
+  log_msg_set_tag_by_name(msg, "korte");
+  log_msg_set_tag_by_name(msg, "citrom");
+
   /* fix some externally or automatically defined values */
-  log_msg_set_host_from(msg, g_strdup("kismacska"), -1);
+  log_msg_set_value(msg, LM_V_HOST_FROM, "kismacska", -1);
   msg->timestamps[LM_TS_RECVD].time.tv_sec = 1139684315;
   msg->timestamps[LM_TS_RECVD].time.tv_usec = 639000;
   msg->timestamps[LM_TS_RECVD].zone_offset = get_local_timezone_ofs(1139684315);
@@ -79,6 +80,7 @@ main(int argc G_GNUC_UNUSED, char *argv[] G_GNUC_UNUSED)
   testcase(msg, "$LEVEL", "err");
   testcase(msg, "$LEVEL_NUM", "3");
   testcase(msg, "$TAG", "9b");
+  testcase(msg, "$TAGS", "alma,korte,citrom");
   testcase(msg, "$PRI", "155");
   testcase(msg, "$DATE", "Feb 11 10:34:56.000");
   testcase(msg, "$FULLDATE", "2006 Feb 11 10:34:56.000");
@@ -157,7 +159,7 @@ main(int argc G_GNUC_UNUSED, char *argv[] G_GNUC_UNUSED)
   testcase(msg, "$MSGONLY", "árvíztűrőtükörfúrógép");
   testcase(msg, "$MESSAGE", "syslog-ng[23323]: árvíztűrőtükörfúrógép");
   testcase(msg, "$SOURCEIP", "10.10.10.10");
-  testcase(msg, "$PROGRAM/var/log/messages/$HOST/$HOST_FROM/$MONTH$DAY$QQQQQvalami", "syslog-ng/var/log/messages/bzorp/kismacska/0211valami");
+  testcase(msg, "$PROGRAM/var/log/messages/$HOST/$HOST_FROM/$MONTH$DAY${QQQQQ}valami", "syslog-ng/var/log/messages/bzorp/kismacska/0211valami");
   testcase(msg, "${APP.VALUE}", "value");
   testcase(msg, "${APP.VALUE:-ures}", "value");
   testcase(msg, "${APP.VALUE2:-ures}", "ures");
@@ -169,8 +171,8 @@ main(int argc G_GNUC_UNUSED, char *argv[] G_GNUC_UNUSED)
   testcase(msg, "$MSG", "árvíztűrőtükörfúrógép");
   testcase(msg, "$MESSAGE", "árvíztűrőtükörfúrógép");
 
-  log_msg_unref(msg);  
-  
+  log_msg_unref(msg);
+
   msg_str = "syslog-ng: árvíztűrőtükörfúrógép [pid test]";
   msg = log_msg_new(msg_str, strlen(msg_str), g_sockaddr_inet_new("10.10.10.10", 1010), 0, NULL, -1, 0xFFFF);
 
@@ -185,7 +187,7 @@ main(int argc G_GNUC_UNUSED, char *argv[] G_GNUC_UNUSED)
   testcase(msg, "$MSGHDR", "syslog-ng[23323]:");
   log_msg_unref(msg);
 
-  msg_str = "<132>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog 3535 ID47 [exampleSDID@0 iut=\"3\" eventSource=\"Application\" eventID=\"1011\"][examplePriority@0 class=\"high\"] BOMAn application event log entry..."; 
+  msg_str = "<132>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog 3535 ID47 [exampleSDID@0 iut=\"3\" eventSource=\"Application\" eventID=\"1011\"][examplePriority@0 class=\"high\"] BOMAn application event log entry...";
   msg = log_msg_new(msg_str, strlen(msg_str), g_sockaddr_inet_new("10.10.10.10", 1010), LP_SYSLOG_PROTOCOL, NULL, -1, 0xFFFF);
 
   testcase(msg, "$PRI", "132");
@@ -198,9 +200,9 @@ main(int argc G_GNUC_UNUSED, char *argv[] G_GNUC_UNUSED)
   testcase(msg, "${.SDATA.exampleSDID@0.eventID}", "1011");
   testcase(msg, "${.SDATA.examplePriority@0.class}", "high");
 
-  
+
   app_shutdown();
-  
+
   if (success)
     return 0;
   return 1;
